#!/usr/bin/python

#    cidalert - CallerID notifications for VOIP phones
#    Copyright (C) 2012  Paul Dwerryhouse
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import bottle
import pygtk
pygtk.require('2.0')
import pynotify
import sys
import os
import atexit
from signal import signal, SIGTERM
import ConfigParser

@bottle.route('/call')
def call():
    global config
    global directory
    global allowed_addrs

    try:
        remote_addr = bottle.request.environ.get('REMOTE_ADDR')

        allowed = False
        for addr in allowed_addrs:
            if (remote_addr == addr):
                allowed = True;
                break;

        if not allowed:
            return

        callerid = bottle.request.GET['callerid']
        name = directory.get(callerid)
        if (name == None):
            name = "Unknown"

        n = pynotify.Notification("Incoming call from:", "{0} ({1})".format(name,callerid))

        if not n.show():
            print "Failed to send notification"
            sys.exit(1)

    except KeyError:
        pass

@bottle.error(404)
def error404(error):
    return 'Nothing here'

def createcfgfile(cfgfile):
    config = ConfigParser.RawConfigParser()
    config.add_section('general')
    config.set('general','port','8998')
    config.set('general','host','localhost')
    config.set('general','allowed_addrs','127.0.0.1')
    config.add_section('directory')
    config.set('directory','0491570156','Mulder')
    config.set('directory','0491570157','Scully')
    with open(cfgfile, 'wb') as configfile:
        config.write(configfile)

def server(port=8998, host="localhost"):
    bottle.run(host=host, port=port, server=bottle.WSGIRefServer,quiet=True)

def shutdown():
    global pidfile
    os.unlink(pidfile)

def cleanup():
    home = os.environ.get('HOME')
    pidfile = home + "/.cidalert/cidalert.pid"
    try:
        os.unlink(pidfile)
    except IOError:
        pass

def status():
    home = os.environ.get('HOME')
    try:
        pidfile = home + "/.cidalert/cidalert.pid"
        p = open(pidfile,'r')
        pid = int(p.read())
        p.close()
        print "cidalert running with pid {0}".format(pid)
    except IOError:
        print "cidalert not running"

def stop():
    home = os.environ.get('HOME')
    pidfile = home + "/.cidalert/cidalert.pid"
    try:
        p = open(pidfile,'r')
        pid = int(p.read())
        p.close()
        os.kill(pid,SIGTERM)
    except IOError:
        pass

def start():
    global config
    global directory
    global pidfile
    global allowed_addrs
    directory = {}

    if not pynotify.init("Basics"):
        sys.exit(1)

    home = os.environ.get('HOME')
    cfgdir = home + "/.cidalert"
    cfgfile = home + "/.cidalert/cidalert.cfg"
    pidfile = home + "/.cidalert/cidalert.pid"

    if not os.path.exists(cfgdir):
        os.mkdir(cfgdir)

    if not os.path.isdir(cfgdir):
        print "{0} is not a directory".format(cfgdir)
        sys.exit(1)
    
    if os.path.exists(pidfile):
        print "cidalert is already running"
        print "If you know that this is incorrect, then run:  cidalert cleanup"
        sys.exit(1)

    if not os.path.exists(cfgfile):
        createcfgfile(cfgfile)

    if not os.path.isfile(cfgfile):
        print "{0} is not a file".format(cfgfile)
        sys.exit(1)
    
    config = ConfigParser.SafeConfigParser()
    config.readfp(open(cfgfile))
    host = config.get('general','host')
    port = config.getint('general','port')
    allowed_addrs = config.get('general','allowed_addrs').split(',')

    for (number,name) in config.items('directory'):
        directory[number] = name
    
    if os.fork() == 0:
        f = open(pidfile,'w')
        f.write(str(os.getpid()))
        f.close()
        atexit.register(shutdown)
        signal(SIGTERM, lambda signum, stack_frame: exit(1))
        server(host=host,port=port)

if __name__ == "__main__":
    start()

